@@ -26,17 +26,30 @@ def expand_resource(group, path):
     return None
 
 
-def expand_wildcard(group, path):
+def expand_wildcard(group, path, **kwargs):
     """Get real path of a resource."""
-
     bld = group.context
     # replacement pattern, {_N} will be replaced with group name of level N
     path = path.format(**group.get_patterns())
     if '*' in path or '?' in path:
+        # waf counts maxdepth from the root directory but we want it relative
+        # to the wildcard template
+        maxdepth = kwargs.get('maxdepth', 25)
+        depth = path.rstrip(os.path.sep).count(os.path.sep)
         if os.path.isabs(path):
-            return [node.abspath() for node in\
-                    bld.root.ant_glob(path.lstrip('/'))]
-        return [node.abspath() for node in bld.path.ant_glob(path)]
+            files = [node.abspath() for node in\
+                    bld.root.ant_glob(path.lstrip('/'),
+                    maxdepth=depth + maxdepth)]
+        else:
+            depth += bld.out_dir.count(os.path.sep) + 1
+            files = [node.abspath() for node in bld.path.ant_glob(path,
+                    maxdepth=depth + maxdepth)]
+        if path.endswith(os.path.sep):
+            dirnames = set()
+            for filename in files:
+                dirnames.add(os.path.dirname(filename))
+            files = list(dirnames)
+        return files
 
     path = os.path.expanduser(path)
     if os.path.isabs(path):
